@@ -1,17 +1,14 @@
-##################################
-#
-# image_tools.py
-# by Gerd Duscher, UTK
-# part of pyTEMlib
-# MIT license except where stated differently
-#
-###############################
+"""
+image_tools.py
+by Gerd Duscher, UTK
+part of pyTEMlib
+MIT license except where stated differently
+"""
+
 import numpy as np
 
 import matplotlib as mpl
 import matplotlib.pylab as plt
-from matplotlib.patches import Polygon  # plotting of polygons -- graph rings
-
 import matplotlib.widgets as mwidgets
 from matplotlib.widgets import RectangleSelector
 
@@ -63,32 +60,113 @@ def get_wavelength(e0):
     """
     Calculates the relativistic corrected de Broglie wave length of an electron
 
-    Input:
-    ------
-        acceleration voltage in volt
-    Output:
+    Parameters
+    ----------
+    e0: float
+      acceleration voltage in volt
+
+    Returns
     -------
-        wave length in 1/nm
+    wave length in 1/nm
     """
 
     eV = const.e * e0
     return const.h/np.sqrt(2*const.m_e*eV*(1+eV/(2*const.m_e*const.c**2)))*10**9
 
 
+def read_image_info(dataset):
+    """Updates dataset.metadata['experiment'] with essential information read from original metadata
+
+    This depends on whether it is originally a nion or a dm3 file
+    """
+    if not isinstance(dataset, sidpy.Dataset):
+        raise TypeError("we need a sidpy.Dataset")
+
+    if 'metadata' in dataset.original_metadata:
+        if 'hardware_source' in dataset.original_metadata['metadata']:
+            experiment_dictionary = read_nion_image_info(dataset.original_metadata)
+    if 'DM' in dataset.original_metadata:
+        experiment_dictionary = read_dm3_image_info(dataset.original_metadata)
+    if 'experiment' not in dataset.metadata:
+        dataset.metadata['experiment'] = {}
+    dataset.metadata['experiment'].update(experiment_dictionary)
+
+
+def read_dm3_image_info(original_metadata):
+    """Read essential parameter from original_metadata originating from a dm3 file"""
+    if not isinstance(original_metadata, dict):
+        raise TypeError('We need a dictionary to read')
+
+    if 'DM' not in original_metadata:
+        return {}
+    main_image = original_metadata['DM']['chosen_image']
+    exp_dictionary = original_metadata['ImageList'][str(main_image)]['ImageTags']
+    experiment = {}
+
+    if 'Acquisition' in exp_dictionary:
+        if 'Parameters' in exp_dictionary['Acquisition']:
+            if 'High Level' in exp_dictionary['Acquisition']['Parameters']:
+                if 'Exposure (s)' in exp_dictionary['Acquisition']['Parameters']['High Level']:
+                    experiment['exposure_time'] = exp_dictionary['Acquisition']['Parameters']['High Level'][
+                        'Exposure (s)']
+
+    if 'Microscope Info' in exp_dictionary:
+        if 'Microscope' in exp_dictionary['Microscope Info']:
+            experiment['microscope'] = exp_dictionary['Microscope Info']['Microscope']
+        if 'Voltage' in exp_dictionary['Microscope Info']:
+            experiment['acceleration_voltage'] = exp_dictionary['Microscope Info']['Voltage']
+        if 'Illumination Mode' in exp_dictionary['Microscope Info']:
+            if exp_dictionary['Microscope Info']['Illumination Mode'] == 'TEM':
+                experiment['convergence_angle'] = 0.0
+                experiment['collection_angle'] = 100.0
+            if exp_dictionary['Microscope Info']['Illumination Mode'] == 'SPOT':
+                experiment['convergence_angle'] = 10.0
+                experiment['collection_angle'] = 50.0
+    return experiment
+
+
+def read_nion_image_info(original_metadata):
+    """Read essential parameter from original_metadata originating from a dm3 file"""
+    if not isinstance(original_metadata, dict):
+        raise TypeError('We need a dictionary to read')
+    if 'metadata' not in original_metadata:
+        return {}
+    if 'hardware_source' not in original_metadata['metadata']:
+        return {}
+    if 'ImageScanned' not in original_metadata['metadata']['hardware_source']:
+        return {}
+
+    exp_dictionary = original_metadata['metadata']['hardware_source']['ImageScanned']
+    experiment = {}
+    print(exp_dictionary)
+    if 'autostem' in exp_dictionary:
+        print('auto')
+
+    print(exp_dictionary.keys())
+    print()
+
+    print(experiment)
+
+
 def fourier_transform(dset):
     """
         Reads information into dictionary 'tags', performs 'FFT', and provides a smoothed FT and reciprocal
         and intensity limits for visualization.
 
-        Input
-        -----
+        Parameters
+        ----------
         dset: sidpy.Dataset
+            image
 
-        Usage
-        -----
+        Returns
+        -------
+        fft_dset: sidpy.Dataset
+            Fourier transform with correct dimensions
 
-        fft_dataset = fourier_transform(sidpy_dataset)
-        fft+dataset.plot()
+        Example
+        -------
+        >>> fft_dataset = fourier_transform(sidpy_dataset)
+        >>> fft_dataset.plot()
     """
 
     assert isinstance(dset, sidpy.Dataset), 'Expected a sidpy Dataset'
@@ -109,14 +187,14 @@ def fourier_transform(dset):
         if len(image_dim) != 2:
             raise ValueError('need at least two SPATIAL dimension for an image stack')
         image_stack = np.squeeze(np.array(dset)[selection])
-        image = np.sum(np.array(image_stack), axis=stack_dim)
+        new_image = np.sum(np.array(image_stack), axis=stack_dim)
     elif dset.data_type == sidpy.DataType.IMAGE:
-        image = np.array(dset)
+        new_image = np.array(dset)
     else:
         return
 
-    image = image - image.min()
-    fft_transform = (np.fft.fftshift(np.fft.fft2(image)))
+    new_image = new_image - new_image.min()
+    fft_transform = (np.fft.fftshift(np.fft.fft2(new_image)))
 
     image_dims = get_image_dims(dset)
 
@@ -130,11 +208,13 @@ def fourier_transform(dset):
     fft_dset.source = dset.title
     fft_dset.modality = 'fft'
 
-    fft_dset.set_dimension(0, sidpy.Dimension(np.fft.fftshift(np.fft.fftfreq(image.shape[0], d=get_slope(dset.x.values))),
+    fft_dset.set_dimension(0, sidpy.Dimension(np.fft.fftshift(np.fft.fftfreq(new_image.shape[0],
+                                                                             d=get_slope(dset.x.values))),
 
                                               name='u', units=units_x, dimension_type='RECIPROCAL',
                                               quantity='reciprocal_length'))
-    fft_dset.set_dimension(1, sidpy.Dimension(np.fft.fftshift(np.fft.fftfreq(image.shape[1], d=get_slope(dset.y.values))),
+    fft_dset.set_dimension(1, sidpy.Dimension(np.fft.fftshift(np.fft.fftfreq(new_image.shape[1],
+                                                                             d=get_slope(dset.y.values))),
                                               name='v', units=units_y, dimension_type='RECIPROCAL',
                                               quantity='reciprocal_length'))
 
@@ -145,21 +225,20 @@ def power_spectrum(dset, smoothing=3):
     """
     Calculate power spectrum
 
-    Input:
-    ------
-            channel: channnel in h5f file with image content
+    Parameters
+    ----------
+    dset: sidpy.Dataset
+        image
+    smoothing: int
+        Gaussian smoothing
 
-    Output:
+    Returns
     -------
-            tags: dictionary with
-                ['data']: fourier transformed image
-                ['axis']: scale of reciprocal image
-                ['power_spectrum']: power_spectrum
-                ['FOV']: field of view for extent parameter in plotting
-                ['minimum_intensity']: suggested minimum intensity for plotting
-                ['maximum_intensity']: suggested maximum intensity for plotting
+    power_spec: sidpy.Dataset
+        power spectrum with correct dimensions
 
     """
+
     fft_transform = fourier_transform(dset)
     fft_mag = np.abs(fft_transform)
     fft_mag2 = ndimage.gaussian_filter(fft_mag, sigma=(smoothing, smoothing), order=0)
@@ -167,8 +246,7 @@ def power_spectrum(dset, smoothing=3):
     power_spec = fft_transform.like_data(np.log(1.+fft_mag2))
 
     # prepare mask
-
-    x, y = np.meshgrid(power_spec.u.values, power_spec.v.values)
+    x, y = np.meshgrid(power_spec.v.values, power_spec.u.values)
     mask = np.zeros(power_spec.shape)
 
     mask_spot = x ** 2 + y ** 2 > 1 ** 2
@@ -178,33 +256,34 @@ def power_spectrum(dset, smoothing=3):
 
     mask[np.where(mask == 1)] = 0  # just in case of overlapping disks
 
-    # minimum_intensity = np.log2(1 + fft_mag2)[np.where(mask == 2)].min() * 0.95
-    # maximum_intensity = np.log2(1 + fft_mag2)[np.where(mask == 2)].max() * 1.05
-    power_spec.metadata = {'smoothing': smoothing}
-    #                       'minimum_intensity': minimum_intensity, 'maximum_intensity': maximum_intensity}
+    minimum_intensity = np.array(power_spec)[np.where(mask == 2)].min() * 0.95
+    maximum_intensity = np.array(power_spec)[np.where(mask == 2)].max() * 1.05
+    power_spec.metadata = {'fft': {'smoothing': smoothing,
+                                   'minimum_intensity': minimum_intensity, 'maximum_intensity': maximum_intensity}}
     power_spec.title = 'power spectrum ' + power_spec.source
 
     return power_spec
 
 
 def diffractogram_spots(dset, spot_threshold):
-    """
-    Find spots in diffractogram and sort them by distance from center
+    """Find spots in diffractogram and sort them by distance from center
 
-    Input:
-    ------
-            fft_tags: dictionary with
-                ['spatial_***']: information of scale of fourier pattern
-                ['data']: power_spectrum
-            spot_threshold: threshold for blob finder
-    Output:
+    Uses blob_log from scipy.spatial
+
+    Parameters
+    ----------
+    dset: sidpy.Dataset
+        diffractogram
+    spot_threshold: float
+        threshold for blob finder
+
+    Returns
     -------
-            spots: numpy array with sorted position (x,y) and radius (r) of all spots
-
+    spots: numpy array
+        sorted position (x,y) and radius (r) of all spots
     """
-    #
 
-# spot detection (for future reference there is no symmetry assumed here)
+    # spot detection (for future reference there is no symmetry assumed here)
     data = np.array(np.log(1+np.abs(dset)))
     data = (data - data.min())
     data = data/data.max()
@@ -233,32 +312,36 @@ def adaptive_fourier_filter(dset, spots, low_pass=3, reflection_radius=0.3):
     """
     Use spots in diffractogram for a Fourier Filter
 
-    Input:
-    ------
-            image:  image to be filtered
-            tags: dictionary with
-                ['spatial_***']: information of scale of fourier pattern
-                ['spots']: sorted spots in diffractogram in 1/nm
-            low_pass: low pass filter in center of diffractogrm
+    Parameters
+    ----------
+    dset: sidpu.Dataset
+        image to be filtered
+    spots: np.ndarray(N,2)
+        sorted spots in diffractogram in 1/nm
+    low_pass:  float
+        low pass filter in center of diffractogram in 1/nm
+    reflection_radius:  float
+        radius of masked reflections in 1/nm
 
     Output:
     -------
             Fourier filtered image
     """
-    # prepare mask
 
+    if not isinstance(dset, sidpy.Dataset):
+        raise TypeError('We need a sidpy.Dataset')
     fft_transform = fourier_transform(dset)
+
+    # prepare mask
     x, y = np.meshgrid(fft_transform.u.values, fft_transform.v.values)
     mask = np.zeros(dset.shape)
 
     # mask reflections
-    # reflection_radius = 0.3 # in 1/nm
     for spot in spots:
         mask_spot = (x - spot[0]) ** 2 + (y - spot[1]) ** 2 < reflection_radius ** 2  # make a spot
         mask = mask + mask_spot  # add spot to mask
 
     # mask zero region larger (low-pass filter = intensity variations)
-    # low_pass = 3 # in 1/nm
     mask_spot = x ** 2 + y ** 2 < low_pass ** 2
     mask = mask + mask_spot
     mask[np.where(mask > 1)] = 1
@@ -269,11 +352,12 @@ def adaptive_fourier_filter(dset, spots, low_pass=3, reflection_radius=0.3):
     filtered_image.source = dset.title
     filtered_image.metadata = {'analysis': 'adaptive fourier filtered', 'spots': spots,
                                'low_pass': low_pass, 'reflection_radius': reflection_radius}
-
     return filtered_image
 
 
 def rotational_symmetry_diffractogram(spots):
+    """ Test rotational symmetry of diffraction spots"""
+
     rotation_symmetry = []
     for n in [2, 3, 4, 6]:
         cc = np.array(
@@ -295,7 +379,31 @@ def rotational_symmetry_diffractogram(spots):
 
 
 def complete_registration(main_dataset, storage_channel=None):
-    """Rigid and then non-rigid (demon) registration"""
+    """Rigid and then non-rigid (demon) registration
+
+    Performs rigid and then non-rigid registration, please see individual functions:
+    - rigid_registration
+    - demon_registration
+
+    Parameters
+    ----------
+    main_dataset: sidpy.Dataset
+        dataset of data_type 'IMAGE_STACK' to be registered
+    storage_channel: h5py.Group
+        optional - location in hdf5 file to store datasets
+
+    Returns
+    -------
+    non_rigid_registered: sidpy.Dataset
+    rigid_registered_dataset: sidpy.Dataset
+
+    """
+
+    if not isinstance(main_dataset, sidpy.Dataset):
+        raise TypeError('We need a sidpy.Dataset')
+    if main_dataset.data_type.name != 'IMAGE_STACK':
+        raise TypeError('Registration makes only sense for an image stack')
+
     rigid_registered_dataset = rigid_registration(main_dataset)
     if storage_channel is None:
         storage_channel = main_dataset.h5_dataset.parent.parent
@@ -314,9 +422,12 @@ def demon_registration(dataset, verbose=False):
     """
     Diffeomorphic Demon Non-Rigid Registration
 
-    Usage:
-    ------
-    dem_reg = demon_reg(cube, verbose = False)
+    Depends on:
+        simpleITK and numpy
+    Please Cite: http://www.simpleitk.org/SimpleITK/project/parti.html
+    and T. Vercauteren, X. Pennec, A. Perchant and N. Ayache
+    Diffeomorphic Demons Using ITK\'s Finite Difference Solver Hierarchy
+    The Insight Journal, http://hdl.handle.net/1926/510 2007
 
     Parameters
     ----------
@@ -328,15 +439,16 @@ def demon_registration(dataset, verbose=False):
     -------
         dem_reg: stack of images with non-rigid registration
 
-    Depends on:
-        simpleITK and numpy
-
-    Please Cite: http://www.simpleitk.org/SimpleITK/project/parti.html
-    and T. Vercauteren, X. Pennec, A. Perchant and N. Ayache
-    Diffeomorphic Demons Using ITK\'s Finite Difference Solver Hierarchy
-    The Insight Journal, http://hdl.handle.net/1926/510 2007
+    Example
+    -------
+    dem_reg = demon_reg(stack_dataset, verbose=False)
     """
 
+    if not isinstance(dataset, sidpy.Dataset):
+        raise TypeError('We need a sidpy.Dataset')
+    if dataset.data_type.name != 'IMAGE_STACK':
+        raise TypeError('Registration makes only sense for an image stack')
+
     dem_reg = np.zeros(dataset.shape)
     nimages = dataset.shape[0]
     if verbose:
@@ -381,7 +493,6 @@ def demon_registration(dataset, verbose=False):
         resampler.SetTransform(out_tx)
         out = resampler.Execute(moving)
         dem_reg[i, :, :] = sITK.GetArrayFromImage(out)
-        # print('image ', i)
 
     if QT_available:
         progress.close()
@@ -407,20 +518,26 @@ def demon_registration(dataset, verbose=False):
 def rigid_registration(dataset):
     """
     Rigid registration of image stack with sub-pixel accuracy
-    used phase_cross_correlation from skimage.registration
+
+    Uses phase_cross_correlation from skimage.registration
     (we determine drift from one image to next)
 
     Parameters
     ----------
-    dataset: sidpy.Datset
+    dataset: sidpy.Dataset
         sidpy dataset with image_stack dataset
 
     Returns
     -------
-    Registered Stack and drift (with respect to center image)
-
+    rigid_registered: sidpy.Dataset
+        Registered Stack and drift (with respect to center image)
     """
 
+    if not isinstance(dataset, sidpy.Dataset):
+        raise TypeError('We need a sidpy.Dataset')
+    if dataset.data_type.name != 'IMAGE_STACK':
+        raise TypeError('Registration makes only sense for an image stack')
+
     nopix = dataset.shape[1]
     nopiy = dataset.shape[2]
     nimages = dataset.shape[0]
@@ -473,16 +590,24 @@ def rigid_registration(dataset):
 
 
 def rig_reg_drift(dset, rel_drift):
-    """
-    Uses relative drift to shift images ontop of each other
-    Shifting is done with shift routine of ndimage from scipy
+    """ Shifting images on top of each other
 
-    is used by Rigid_Registration routine
+    Uses relative drift to shift images on top of each other,
+    with center image as reference.
+    Shifting is done with shift routine of ndimage from scipy.
+    This function is used by rigid_registration routine
 
-    Input image_channel with image_stack numpy array
-    relative_drift from image to image as list of [shiftx, shifty]
+    Parameters
+    ----------
+    dset: sidpy.Dataset
+        dataset with image_stack
+    rel_drift:
+        relative_drift from image to image as list of [shiftx, shifty]
 
-    output stack and drift
+    Returns
+    -------
+    stack: numpy array
+    drift: list of drift in pixel
     """
 
     rig_reg = np.zeros(dset.shape)
@@ -502,9 +627,19 @@ def rig_reg_drift(dset, rel_drift):
 
 
 def crop_image_stack(rig_reg, drift):
+    """Crop images in stack according to drift
+
+    This function is used by rigid_registration routine
+
+    Parameters
+    ----------
+    rig_reg: numpy array (N,x,y)
+    drift: list (2,B)
+
+    Returns:
+    numpy array
     """
-    ## Crop images
-    """
+
     xpmin = int(-np.floor(np.min(np.array(drift)[:, 0])))
     xpmax = int(rig_reg.shape[1] - np.ceil(np.max(np.array(drift)[:, 0])))
     ypmin = int(-np.floor(np.min(np.array(drift)[:, 1])))
@@ -514,6 +649,8 @@ def crop_image_stack(rig_reg, drift):
 
 
 class ImageWithLineProfile:
+    """Image with line profile"""
+
     def __init__(self, data, extent, title=''):
         fig, ax = plt.subplots(1, 1)
         self.figure = fig
@@ -532,68 +669,66 @@ class ImageWithLineProfile:
     def __call__(self, event):
         if event.inaxes != self.line.axes:
             return
-        self.start_x = self.end_x 
+        self.start_x = self.end_x
         self.start_y = self.end_y
-        
+
         self.line.set_data([self.start_x, event.xdata], [self.start_y, event.ydata])
         self.line.figure.canvas.draw()
-        
+
         self.end_x = event.xdata
         self.end_y = event.ydata
-        
+
         self.update()
-        
+
     def update(self):
-        
         if not self.line_plot:
             self.line_plot = True
             self.figure.clear()
             self.ax = self.figure.subplots(2, 1)
             self.ax[0].imshow(self.data, extent=self.extent)
             self.ax[0].set_title(self.title)
-        
+
             self.line,  = self.ax[0].plot([0], [0], color='orange')  # empty line
             self.line_plot, = self.ax[1].plot([], [], color='orange')
             self.ax[1].set_xlabel('distance [nm]')
-        
+
         x0 = self.start_x
         x1 = self.end_x
         y0 = self.start_y
         y1 = self.end_y
         length_plot = np.sqrt((x1-x0)**2+(y1-y0)**2)
-       
+
         num = length_plot*(self.data.shape[0]/self.extent[1])
         x = np.linspace(x0, x1, num)*(self.data.shape[0]/self.extent[1])
         y = np.linspace(y0, y1, num)*(self.data.shape[0]/self.extent[1])
-        
+
         # Extract the values along the line, using cubic interpolation
         zi2 = ndimage.map_coordinates(self.data.T, np.vstack((x, y)))
-        
+
         x_axis = np.linspace(0, length_plot, len(zi2))
-        
         self.x = x_axis
         self.z = zi2
-        
+
         self.line_plot.set_xdata(x_axis)
         self.line_plot.set_ydata(zi2)
         self.ax[1].set_xlim(0, x_axis.max())
         self.ax[1].set_ylim(zi2.min(), zi2.max())
         self.ax[1].draw()
-        
+
 
 def histogram_plot(image_tags):
+    """interactive histogram"""
     nbins = 75
     color_map_list = ['gray', 'viridis', 'jet', 'hot']
-    
-    if 'minimum_intensity' not in image_tags: 
+    if 'minimum_intensity' not in image_tags:
         image_tags['minimum_intensity'] = image_tags['plotimage'].min()
     minimum_intensity = image_tags['minimum_intensity']
-    if 'maximum_intensity' not in image_tags: 
+    if 'maximum_intensity' not in image_tags:
         image_tags['maximum_intensity'] = image_tags['plotimage'].max()
     data = image_tags['plotimage']
     vmin = image_tags['minimum_intensity']
     vmax = image_tags['maximum_intensity']
-    if 'color_map' not in image_tags: 
+    if 'color_map' not in image_tags:
         image_tags['color_map'] = color_map_list[0]
     cmap = plt.cm.get_cmap(image_tags['color_map'])
 
@@ -605,7 +740,6 @@ def histogram_plot(image_tags):
     width = bin_edges[1]-bin_edges[0]
 
     def onselect(vmin, vmax):
-        
         ax1.clear()
         cmap = plt.cm.get_cmap(image_tags['color_map'])
 
@@ -628,13 +762,13 @@ def histogram_plot(image_tags):
 
         image_tags['minimum_intensity'] = vmin
         image_tags['maximum_intensity'] = vmax
-        
+
     def onclick(event):
         global event2
         event2 = event
         print('%s click: button=%d, x=%d, y=%d, xdata=%f, ydata=%f' %
               ('double' if event.dblclick else 'single', event.button,
-               event.x, event.y, event.xdata, event.ydata)) 
+               event.x, event.y, event.xdata, event.ydata))
         if event.inaxes == ax1:
             if event.button == 3:
                 ind = color_map_list.index(image_tags['color_map'])+1
@@ -666,16 +800,15 @@ def histogram_plot(image_tags):
 
 
 def clean_svd(im, pixel_size=1, source_size=5):
+    """De-noising of image by using first component of single value decomposition"""
     patch_size = int(source_size/pixel_size)
     if patch_size < 3:
         patch_size = 3
-    print(patch_size)
-    
     patches = image.extract_patches_2d(im, (patch_size, patch_size))
     patches = patches.reshape(patches.shape[0], patches.shape[1]*patches.shape[2])
-    
+
     num_components = 32
-    
+
     u, s, v = randomized_svd(patches, num_components)
     u_im_size = int(np.sqrt(u.shape[0]))
     reduced_image = u[:, 0].reshape(u_im_size, u_im_size)
@@ -686,69 +819,99 @@ def clean_svd(im, pixel_size=1, source_size=5):
 def rebin(im, binning=2):
     """
     rebin an image by the number of pixels in x and y direction given by binning
-    
-    Input:
-    ------
-            image: numpy array in 2 dimensions
-    
-    Output:
+
+    Parameter
+    ---------
+    image: numpy array in 2 dimensions
+
+    Returns
     -------
-            binned image 
+    binned image as numpy array
     """
     if len(im.shape) == 2:
         return im.reshape((im.shape[0]//binning, binning, im.shape[1]//binning, binning)).mean(axis=3).mean(1)
     else:
-        print('not a 2D image')
+        raise TypeError('not a 2D image')
         return im
 
 
 def cart2pol(points):
     """Cartesian to polar coordinate conversion
 
-    :param points: points to be converted
-    :type points: numpy  array
+    Parameters
+    ---------
+    points: float or numpy array
+        points to be converted (Nx2)
 
-    :returns: distance and angle
-    :rtype: numpy arrays
+    Returns
+    -------
+    rho: float or numpy array
+        distance
+    phi: float or numpy array
+        angle
     """
+
     rho = np.linalg.norm(points[:, 0:2], axis=1)
     phi = np.arctan2(points[:, 1], points[:, 0])
     return rho, phi
 
 
 def pol2cart(rho, phi):
+    """Polar to Cartesian coordinate conversion
+
+    Parameters
+    ----------
+    rho: float or numpy array
+        distance
+    phi: float or numpy array
+        angle
+
+    Returns
+    -------
+    x: float or numpy array
+        x coordinates of converted points(Nx2)
+    """
+
     x = rho * np.cos(phi)
     y = rho * np.sin(phi)
     return x, y
 
 
 def xy2polar(points, rounding=1e-3):
-    """
-    Conversion from carthesian to polar coordinates
-    
+    """ Conversion from carthesian to polar coordinates
+
     the angles and distances are sorted by r and then phi
     The indices of this sort is also returned
-    
-    input points: numpy array with number of points in axis 0 first two elements in axis 1 are x and y
-    
-    optional rounding in significant digits 
-    
-    returns r,phi, sorted_indices
+
+    Parameters
+    ----------
+    points: numpy array
+        number of points in axis 0 first two elements in axis 1 are x and y
+    rounding: int
+        optional rounding in significant digits
+
+    Returns
+    -------
+    r, phi, sorted_indices
     """
-    
+
     r, phi = cart2pol(points)
-    
+
     phi = phi-phi.min()  # only positive angles
     r = (np.floor(r/rounding))*rounding  # Remove rounding error differences
 
     sorted_indices = np.lexsort((phi, r))  # sort first by r and then by phi
     r = r[sorted_indices]
     phi = phi[sorted_indices]
-    
+
     return r, phi, sorted_indices
-            
+
 
 def cartesian2polar(x, y, grid, r, t, order=3):
+    """Transform cartesian grid to polar grid
+
+    Used by warp
+    """
 
     rr, tt = np.meshgrid(r, t)
 
@@ -765,6 +928,8 @@ def cartesian2polar(x, y, grid, r, t, order=3):
 
 
 def warp(diff, center):
+    """Convert diffraction pattern to polar coordinates"""
+
     # Define original polar grid
     nx = diff.shape[0]
     ny = diff.shape[1]
@@ -784,7 +949,24 @@ def warp(diff, center):
 
 def calculate_ctf(wavelength, cs, defocus, k):
     """ Calculate Contrast Transfer Function
+
     everything in nm
+
+    Parameters
+    ----------
+    wavelength: float
+        deBroglie wavelength of electrons
+    cs: float
+        spherical aberration coefficient
+    defocus: float
+        defocus
+    k: numpy array
+        reciprocal scale
+
+    Returns:
+    ctf: numpy array
+        contrast transfer function
+
     """
     ctf = np.sin(np.pi*defocus*wavelength*k**2+0.5*np.pi*cs*wavelength**3*k**4)
     return ctf
@@ -793,13 +975,16 @@ def calculate_ctf(wavelength, cs, defocus, k):
 def calculate_scherzer(wavelength, cs):
     """
     Calculate the Scherzer defocus. Cs is in mm, lambda is in nm
-    # EInput and output in nm
+
+    # Input and output in nm
     """
+
     scherzer = -1.155*(cs*wavelength)**0.5  # in m
     return scherzer
 
 
 def calibrate_image_scale(fft_tags, spots_reference, spots_experiment):
+    """depreciated get change of scale from comparison of spots to Bragg angles """
     gx = fft_tags['spatial_scale_x']
     gy = fft_tags['spatial_scale_y']
 
@@ -820,17 +1005,19 @@ def calibrate_image_scale(fft_tags, spots_reference, spots_experiment):
 
 
 def align_crystal_reflections(spots, crystals):
+    """ Depreciated - use diffraction spots"""
+
     crystal_reflections_polar = []
     angles = []
     exp_r, exp_phi = cart2pol(spots)  # just in polar coordinates
     spots_polar = np.array([exp_r, exp_phi])
-        
+
     for i in range(len(crystals)):
         tags = crystals[i]
         r, phi, indices = xy2polar(tags['allowed']['g'])  # sorted by r and phi , only positive angles
         # we mask the experimental values that are found already
         angle = 0.
-        
+
         angle_i = np.argmin(np.abs(exp_r - r[1]))
         angle = exp_phi[angle_i] - phi[0]
         angles.append(angle)  # Determine rotation angle
@@ -842,7 +1029,7 @@ def align_crystal_reflections(spots, crystals):
             # print(dif.min())
             if dif.min() < 1.5:
                 ind = np.argmin(dif)
-        
+
     return crystal_reflections_polar, angles
 
 
@@ -901,7 +1088,7 @@ def decon_lr(o_image, probe,  verbose=False):
     over_d = 2 * ap_angle / wl
 
     dx = o_image.x[1]-o_image.x[0]
-    dk = 1.0 / float( o_image.x[-1])  # last value of x axis is field of view
+    dk = 1.0 / float(o_image.x[-1])  # last value of x axis is field of view
     screen_width = 1 / dx
 
     aperture = np.ones(o_image.shape, dtype=np.complex64)
@@ -962,8 +1149,7 @@ def decon_lr(o_image, probe,  verbose=False):
         progress.close()
     print('\n Lucy-Richardson deconvolution converged in ' + str(i) + '  Iterations')
     est2 = np.real(fftpack.ifft2(fftpack.fft2(est) * fftpack.fftshift(aperture)))
-    print(est2.shape)
-    out_dataset =  o_image.like_data(est2)
+    out_dataset = o_image.like_data(est2)
     out_dataset.title = 'Lucy Richardson deconvolution'
     out_dataset.data_type = 'image'
     return out_dataset
